// type de carte NodeMCU 1.0 (ESP-12E module)
// syntaxe
// https://(ip_adress)       pour avoir la page d'acceuil de l'application
// https://(ip_adress)/cfg_gen  pour avoir la page d'acceuil des parametres generaux
// https://(ip_adress)/cfg_vol pour avoir la page d'acceuil des parametres du volet (non développé)
// 18/03/2020   V7.1 fixe le probleme de l'heure de descente ou monté (jusqu'a la V7.0, tous les volets prenaient l'heure du premier volet bug dans traite_automatisation (lire_volet activé en dehors de la boucle))
// 20/03/2020   V7.2 change le serveur ntp from ntp.midway.ovh to ntp.ovh.net car ca ne fonctionne plus sur midway. ajoute la date a l'heure donnée sue la page principale
// 22/03/2020   V7.3 interval entre 2 requetes NTP passe de 5 minutes a 1 jours
// 24/03/2020   V7.4 probleme dans traite_automatisation "descente_volet_prog" calculé au mauvais endroit
// 29/03/2020   V7.5 au changement d'heure, on change la valeur de TIMEZONE
// 01/06/2020   V7.6 corrige bug : en cas de coupure de courant la requete ntp est faite alors que la box n'est pas encore fonctionnelle

  // Version 'p' = production , 'd' = developpement
String VERSION = "7.6p";

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
//#include <ESP8266WebServer.h>
#include <ESP8266WebServerSecure.h>
#include <EEPROM.h>
#include <Dusk2Dawn.h>
#include <TimeLib.h>
#include <WiFiUdp.h>

#include "chemin des mots de passe WIFI et admin\Code.h"
#include "chemin de la clef 1024 bits\key_1024.h"
#include "chemin du certificat X509\x509_1024.h"

//#ifndef STASSID
//#define STASSID "xxxxx"
//#define STAPSK  "yyyyy"
//#endif

#define PORT_TX 5 //5 of PORTD = DigitalPin 5
#define SYMBOL 640
#define HAUT 0x2
#define STOP 0x1
#define BAS 0x4
#define PROG 0x8
#define SIZE_REC_VOLET 28

short NB_LOGIN_FAILED = 0;
byte NB_VOLETS_DECLARES;
short OFFSET_VERSUS_SOLEIL;
short OFFSET_MN_SUNSHINE;
short OFFSET_MN_SUNSET;
unsigned short DELAY_TO_STOP_FOR_TYPE_2 = 15000;
unsigned short HEURE_MINI_DESCENTE_VOLETS;
unsigned short HEURE_MAXI_DESCENTE_VOLETS;
unsigned short HEURE_MINI_MONTE_VOLETS;
unsigned short HEURE_MAXI_MONTE_VOLETS;
const char* WIFI_SSID = STASSID;
const char* WIFI_PASSWORD = STAPSK;
const char* USER_WEB = ADMIN_USER;
const char* USER_PAS = ADMIN_PASS;
String IDENT_COOKIE = COOKIE_ID;
unsigned int ADRESSE_TELECOMMANDE;
byte SENS_TELECOMMANDE;
byte TYPE_VOLET; // 0 = somfy, 1 = Bubendorf, 2 = Somfy + stop apres delai de DELAY_TO_STOP_FOR_TYPE_2 millisecondes
byte VALIDATION_PROG_AUTO;
//0 = pas de programation automatiqe autorisée ;
//1 = full automatique autorisé ;
//2 = Descente automatique autorisée seulement
//3 = Montée automatique autorisée seuelement
unsigned short newRollingCode = 11;       //<-- code sur 16 bits ==> pour ESP8266 = short
unsigned short ROLLING_CODE = 0;           //<-- code sur 16 bits ==> pour ESP8266 = short
byte frame[7];
byte checksum;
unsigned int VOLET_NUMBER = 0;
// volet = 0 volet des escaliers
// volet = 1 volet du salon
// volet = 2 volet de la salle à manger
// volet = 4 les 2 volets des 2 chambres

String lecture_serie = "               ";
String lecture2_serie = "               ";

boolean get_set_gen_param(char get_set);

Dusk2Dawn Clermont_Ferrand(45.8402, 3.2143, OFFSET_VERSUS_SOLEIL); // eglise de lussat

int get_Sunset();
int get_Sunrise();
void send_tram_to_volet(byte *frame );
void BuildFrame(byte *frame, byte button);
void SendCommand(byte *frame, byte sync);
void initialize_volet( int VOLET_NUMBER);
void affiche_volet( int VOLET_NUMBER);
void monter_volet(int numero_de_volet);
void descendre_volet(int numero_de_volet);
void stop_volet(int numero_de_volet);
void programme_volet(int numero_de_volet);
void handle_formulaire_cfg_gen();
void handle_Mode_auto();
void handle_Mode_manu();
void handle_cfg_gen();
void handle_cfg_vol();
void handle_increment_volet();
void handle_formulaire_cfg_vol();
void handleRoot();
void handleLogin();
void handleNotFound();
void traite_requete_serie();
void traite_tous_les_volets(int sens_volet);
void check_nom_volet();
int compte_characteres(String lecture_serie);
void lire_volet( int VOLET_NUMBER, boolean );
unsigned int get_unsigned_int_serie (boolean silent);
unsigned short  get_unsigned_short_serie (boolean silent);
short  get_short_serie (boolean silent);
unsigned long  DATE_TRAITE=0;
short  get_signed_short_serie (boolean silent);
boolean get_oui_non_serie(boolean silent);
byte  get_byte_serie (boolean silent);
void read_nom_volet_eeprom ();
void write_nom_volet_eeprom ();
void get_ntp_time();
void construire_line(int cas) ;
void handleSubmit();
void traite_automatisation();
void affiche_menu();
String NOM_DU_VOLET = "              ";
boolean MODE_DEBUG = false;
boolean ERROR_SAISIE = false;
boolean MODE_AUTOMATIQUE = false;
boolean VOLET_TRAITE_MATIN[15];
boolean VOLET_TRAITE_SOIR[15];
int PIN_DESCENTE;
int PIN_MONTE;
boolean PIN_BUBENDORF_DEFINED = false;

// Pour access serveur ntp

//static const char ntpServerName[] = "ntp.crashdump.fr";
//static const char ntpServerName[] = "ntp.midway.ovh";
static const char ntpServerName[] = "ntp.ovh.net";
//static const char ntpServerName[] = "ntp.unice.fr";
WiFiUDP Udp;
unsigned int localPort = 8888;  // local port to listen for UDP packets
void digitalClockDisplay();
void printDigits(int digits);
void sendNTPpacket(IPAddress &address);
time_t prevDisplay = 0; // when the digital clock was displayed
int TIMEZONE = 1;     // Central European Time  (1 en hiver, 2 en ete) ce parametre est mis a jour par l'option de mise a jour des parametres generaux


String PAGE_ACCEUIL;
int VOLET_CHOISI = 0; //  0 = tous , 1 = salon, 2 = escalier , 3 = salle a manger , 4 = 2 chambres
int SENS_CHOISI = 0;  // 0 = descente, 1 = monte

//ESP8266WebServer server(80);
ESP8266WebServerSecure server(443);

//Check if header is present and correct


//-------------------------------------------------------   is_authenticated ------------------------------------------------------------------------------------------------------------------------------

bool is_authenticated() {

  if (server.hasHeader("Cookie")) {
    Serial.print("Found cookie: ");
    String cookie = server.header("Cookie");
    Serial.println(cookie);
    if (cookie.indexOf(IDENT_COOKIE) != -1) {
    //if (cookie.indexOf("ESPSESSIONID=1") != -1) {
      Serial.println("Authentication Successful");
      NB_LOGIN_FAILED=0;
      return true;
    }
  }
  NB_LOGIN_FAILED++;
 
  Serial.print("Authentication Failed : ");Serial.println(NB_LOGIN_FAILED);
  return false;
}

//login page, also called for disconnect



//-------------------------------------------------------   handleLogin ------------------------------------------------------------------------------------------------------------------------------

void handleLogin() {
  String msg;
  if (server.hasHeader("Cookie")) {
    Serial.print("Found cookie: ");
    String cookie = server.header("Cookie");
    Serial.println(cookie);
  }
  if (server.hasArg("DISCONNECT")) {
    Serial.println("Disconnection");
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.sendHeader("Set-Cookie", "ESPSESSIONID=0");
    server.send(301);
    return;
  }
  if (server.hasArg("USERNAME") && server.hasArg("PASSWORD")) {
    if (server.arg("USERNAME") == USER_WEB &&  server.arg("PASSWORD") == USER_PAS && NB_LOGIN_FAILED < 5 ) { // au bout de 4 failure, soit il faut une authent successfull avec un cookie, soit il faut rebooter l'ESP. Pas de message d'erreur au bout de 4 failure, on refuse juste l'authentification sans rien dire
      server.sendHeader("Location", "/");
      server.sendHeader("Cache-Control", "no-cache");
      String Authent_cookie = IDENT_COOKIE +  "; Max-Age=7776000";
      server.sendHeader("Set-Cookie", Authent_cookie); // expiration a  90 jours
 //     server.sendHeader("Set-Cookie", "ESPSESSIONID=1891784; Max-Age=1000000"); // expiration a  90 jours
      server.send(301);
      Serial.println("Log in Successful");
      NB_LOGIN_FAILED=0;
      return;
    }
    NB_LOGIN_FAILED++;
    msg = "Wrong username/password! try again.";
    Serial.print("Log in Failed  : ");Serial.println(NB_LOGIN_FAILED);
  }
  construire_line(0 );
  server.send(200, "text/html", PAGE_ACCEUIL);
}

//-------------------------------------------------------   handleRoot ------------------------------------------------------------------------------------------------------------------------------


//root page can be accessed only if authentication is ok
void handleRoot() {
 

  if (!is_authenticated()) {
    construire_line(1) ;
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }
      
  construire_line(1) ;
  server.send(200, "text/html",PAGE_ACCEUIL);
  handleSubmit();
}

//-------------------------------------------------------   handleNotFound ------------------------------------------------------------------------------------------------------------------------------

void handleNotFound() {

construire_line(0 );
server.send(200, "text/html", PAGE_ACCEUIL);
handleLogin();
}

//-------------------------------------------------------   setup ------------------------------------------------------------------------------------------------------------------------------

void setup(void) {

  Serial.begin(115200);
  EEPROM.begin(512);

  EEPROM.get(510, NB_VOLETS_DECLARES);
  EEPROM.get(499, OFFSET_VERSUS_SOLEIL);
  TIMEZONE=-OFFSET_VERSUS_SOLEIL ; // 2 en été, 1 en hiver

  // initialisation de la pin 5 pour commander le tranmitter
  pinMode(PORT_TX, OUTPUT);
  digitalWrite(PORT_TX, LOW);

  affiche_menu();

  lire_volet(VOLET_NUMBER, true);

  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  Serial.println("");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(WIFI_SSID);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  get_ntp_time();

ERROR_SAISIE = get_set_gen_param('l');

  server.getServer().setServerKeyAndCert_P(rsakey, sizeof(rsakey), x509, sizeof(x509));
  
  server.on("/Mode_automatique", handle_Mode_auto);
  server.on("/Mode_manuel", handle_Mode_manu);
  server.on("/cfg_gen", handle_cfg_gen);
  server.on("/cfg_vol", handle_cfg_vol);
  server.on("/increment_volet",handle_increment_volet);
  server.on("/formulaire_cfg_gen", handle_formulaire_cfg_gen);
  server.on("/formulaire_cfg_vol", handle_formulaire_cfg_vol);

  server.on("/", handleRoot);
  server.on("/login", handleLogin);
  //server.on("/inline", []() {
  //});

  server.onNotFound(handleNotFound);
  //here the list of headers to be recorded
  const char * headerkeys[] = {"User-Agent", "Cookie"} ;
  size_t headerkeyssize = sizeof(headerkeys) / sizeof(char*);
  //ask server to track these headers
  server.collectHeaders(headerkeys, headerkeyssize);
  server.begin();
  Serial.println("HTTP server started");

  delay (500);
}

//-------------------------------------------------------   loop ------------------------------------------------------------------------------------------------------------------------------

void loop(void) {

  // verification qu'on a l'heure (cas d'une coupure electrique, il faut attendre que la box remonte pour avoir l'heure).

  if (year() < 2000 ) 
  {
    Serial.println("20 secondes d'arret pour attendre la box");
    delay (20000);
    
    get_ntp_time();
  }

  // requetes web

  server.handleClient();

  // requetes serie

  if (Serial.available() > 0) {
    traite_requete_serie();

  }
  
    // automatisation

    if (MODE_AUTOMATIQUE == true) {
      int save_volet_number = VOLET_NUMBER;
      traite_automatisation();
      VOLET_NUMBER =save_volet_number;
      lire_volet(VOLET_NUMBER,true);
    }
}

//-------------------------------------------------------   handle_increment_volet ------------------------------------------------------------------------------------------------------------------------------
void handle_increment_volet() {
  
              VOLET_NUMBER ++;
              if (VOLET_NUMBER >= NB_VOLETS_DECLARES){
                VOLET_NUMBER=0;
              }
              lire_volet(VOLET_NUMBER,true);
              construire_line(3);
              server.send(200, "text/html",  PAGE_ACCEUIL);

}
//-------------------------------------------------------   handle_formulaire_cfg_gen ------------------------------------------------------------------------------------------------------------------------------
void handle_formulaire_cfg_gen() {

    if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }

    if (server.args() == 6) {
    String text_nbr_volet = server.arg(0);
    NB_VOLETS_DECLARES = text_nbr_volet.toInt();
    EEPROM.put(510, NB_VOLETS_DECLARES);

    String text_offset_soleil = server.arg(1);
    OFFSET_VERSUS_SOLEIL = text_offset_soleil.toInt();
    EEPROM.put(499, OFFSET_VERSUS_SOLEIL);
    TIMEZONE=-OFFSET_VERSUS_SOLEIL ; // 2 en été, 1 en hiver
    get_ntp_time(); 

    String text_h_min_monte_vol = server.arg(2);
    HEURE_MINI_MONTE_VOLETS  = text_h_min_monte_vol.toInt();
    EEPROM.put(506,HEURE_MINI_MONTE_VOLETS);

    String text_h_max_monte_vol = server.arg(3);
    HEURE_MAXI_MONTE_VOLETS = text_h_max_monte_vol.toInt();
    EEPROM.put(508,HEURE_MAXI_MONTE_VOLETS);
    
    String text_h_min_desce_vol = server.arg(4);
    HEURE_MINI_DESCENTE_VOLETS = text_h_min_desce_vol.toInt();
    EEPROM.put(502,HEURE_MINI_DESCENTE_VOLETS);

    String text_h_max_desce_vol = server.arg(5);
    HEURE_MAXI_DESCENTE_VOLETS = text_h_max_desce_vol.toInt();
    EEPROM.put(504,HEURE_MAXI_DESCENTE_VOLETS);      
      
      
      EEPROM.commit();

  construire_line(1) ;
  server.sendHeader("Location", "/");
  server.send(301);
  //server.send(200, "text/html",PAGE_ACCEUIL);
  handleSubmit();
      
    }
}
//-------------------------------------------------------   handle_formulaire_cfg_vol ------------------------------------------------------------------------------------------------------------------------------
void handle_formulaire_cfg_vol() {

  int ascii_value;
  
    if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }

    if (server.args() == 9) {
    String text_numero_volet = server.arg(0);
    VOLET_NUMBER = text_numero_volet.toInt(); 

    String Str_type_volet = server.arg(1);
    TYPE_VOLET = Str_type_volet.toInt(); 

    int nb_car= server.arg(2).length();
    //Serial.print("nbre de carac :  ");Serial.println(nb_car);
    if (nb_car > 12) 
    {
      nb_car=12;
    }
   
    for (int compteur = 0; compteur <= nb_car; compteur ++)
    {
      NOM_DU_VOLET[compteur] =  server.arg(2)[compteur];
      int toto = NOM_DU_VOLET[compteur];
      Serial.print("web : "); Serial.println(toto);
    }
    for (int compteur =nb_car+1; compteur< 14; compteur ++)
    {
      NOM_DU_VOLET[compteur]=0;
    }

    check_nom_volet();

    String Str_Rolling = server.arg(3);
    ROLLING_CODE = Str_Rolling.toInt(); 
    
    String Str_add_volet = server.arg(4);
    ADRESSE_TELECOMMANDE = Str_add_volet.toInt();

    String Str_sens_commande = server.arg(5);
    SENS_TELECOMMANDE = Str_sens_commande.toInt();

    String Str_automatisation = server.arg(6);
    VALIDATION_PROG_AUTO = Str_automatisation.toInt();
    
    String Str_offset_leve = server.arg(7);
    OFFSET_MN_SUNSHINE = Str_offset_leve.toInt();

    String Str_offset_descente = server.arg(8);
    OFFSET_MN_SUNSET = Str_offset_descente.toInt();
      

    // on peut ecrire le volet
    EEPROM.put( SIZE_REC_VOLET * VOLET_NUMBER, ROLLING_CODE);
    EEPROM.put(2  + SIZE_REC_VOLET * VOLET_NUMBER, ADRESSE_TELECOMMANDE);
    EEPROM.put(6  + SIZE_REC_VOLET * VOLET_NUMBER, SENS_TELECOMMANDE);
    EEPROM.put(7  + SIZE_REC_VOLET * VOLET_NUMBER, TYPE_VOLET);
    EEPROM.put(8  + SIZE_REC_VOLET * VOLET_NUMBER, VALIDATION_PROG_AUTO);
//    EEPROM.put(9 1 byte libre 
    EEPROM.put(10  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSHINE);
    EEPROM.put(12  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSET);
    
    write_nom_volet_eeprom();
    EEPROM.commit();

    affiche_volet(VOLET_NUMBER);
}   
      construire_line(1) ;
      server.sendHeader("Location", "/");
      server.send(301);
      //server.send(200, "text/html",PAGE_ACCEUIL);
      handleSubmit();
      

}
//-------------------------------------------------------   handle_Mode_manu ------------------------------------------------------------------------------------------------------------------------------

void handle_Mode_manu() {
  Serial.print("Mode manuel active");
  String header;
  if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }
    MODE_AUTOMATIQUE=false;
    EEPROM.put(501,MODE_AUTOMATIQUE);
    construire_line(1) ;
    //server.sendHeader("Location", "/");
    server.send(200, "text/html",  PAGE_ACCEUIL);
    //server.send(301);
}

//-------------------------------------------------------   handle_Mode_auto ------------------------------------------------------------------------------------------------------------------------------


//root page can be accessed only if authentication is ok
void handle_Mode_auto() {
  Serial.print("Mode automatique active");
  if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }
  MODE_AUTOMATIQUE=true;
  EEPROM.put(501,MODE_AUTOMATIQUE);
  construire_line(1) ;
  //server.sendHeader("Location", "/");
  server.send(200, "text/html", PAGE_ACCEUIL);
  //server.send(301);
}

//-------------------------------------------------------   handle_cfg_gen ------------------------------------------------------------------------------------------------------------------------------


//root page can be accessed only if authentication is ok
void handle_cfg_gen() {

  if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }
  
  construire_line(2);
  server.send(200, "text/html", PAGE_ACCEUIL);
}
//-------------------------------------------------------   handle_cfg_vol ------------------------------------------------------------------------------------------------------------------------------


//root page can be accessed only if authentication is ok
void handle_cfg_vol() {

  if (!is_authenticated()) {
    server.sendHeader("Location", "/login");
    server.sendHeader("Cache-Control", "no-cache");
    server.send(301);
    return;
  }
  construire_line(3);
  server.send(200, "text/html", PAGE_ACCEUIL);
}
//-------------------------------------------------------   traite_requete_serie ------------------------------------------------------------------------------------------------------------------------------


void traite_requete_serie() {
  char serie;

  lecture_serie = "               ";
  lecture_serie = Serial.readString();
  int nb_char = compte_characteres(lecture_serie);

  serie = lecture_serie[0];

  switch (serie)  {
    case '?':
      affiche_menu();
      break;   
    case 'm':
      monter_volet(VOLET_NUMBER);
      break;
    case 's':
      stop_volet(VOLET_NUMBER);

      break;
    case 'd':
      descendre_volet(VOLET_NUMBER);

      break;
    case 'p':
      programme_volet(VOLET_NUMBER);

      break;
    case  'i':
      initialize_volet(VOLET_NUMBER);
      break;
    case  'l':
      lire_volet( VOLET_NUMBER, false );
      break;
    case 'v':
      lecture_serie[0] = ' ';
      lecture_serie.trim();
      VOLET_NUMBER = lecture_serie.toInt();
      if (VOLET_NUMBER >= NB_VOLETS_DECLARES || NB_VOLETS_DECLARES < 0 ) {
        Serial.println("");
        Serial.print(" --------------> Attention le nombre de volets doit etre compris entre 0 et le nombre de volets declares - 1 dans les parametres generaux. Donc maximum : "); Serial.print(NB_VOLETS_DECLARES-1); Serial.print(" <---------------------------------");
        ERROR_SAISIE = true;
      }
      if (ERROR_SAISIE == false) {
        Serial.println("");
        lire_volet(VOLET_NUMBER, false);
        Serial.println("");
      }
      ERROR_SAISIE = false;
      break;
    case 'g':

        ERROR_SAISIE = get_set_gen_param(lecture_serie[1]);
        ERROR_SAISIE = false;  // clean eror if any
      break;
      case 'h':
        Serial.print("Heure actuelle dans l'ESP8266    :  ");
        digitalClockDisplay();

        char time3[] = "00:00";
        Dusk2Dawn::min2str(time3, get_Sunrise());
        Serial.print("Heure de lever du soleil         :  ");
        Serial.print(get_Sunrise());  Serial.print("   ("); Serial.print(time3); Serial.println("   )");

        char time2[] = "00:00";
        Dusk2Dawn::min2str(time2, get_Sunset());
        Serial.print("Heure de coucher du soleil       :  ");
        Serial.print(get_Sunset());  Serial.print("   ("); Serial.print(time2); Serial.println("   )");
      break;
  }// end case
}


//-------------------------------------------------------   send_tram_to_volet ------------------------------------------------------------------------------------------------------------------------------

void send_tram_to_volet(byte *frame) {

  Serial.println("Envoie la trame");
  SendCommand(frame, 2);
  for (int i = 0; i < 2; i++) {
    SendCommand(frame, 7);
  }
}


//-------------------------------------------------------   compte_characteres ------------------------------------------------------------------------------------------------------------------------------

int  compte_characteres(String chaine_char) {

  int compteur = 0;
  int resultat = 0;
  char  serie;
  while ( compteur < 10) {
    serie = chaine_char[compteur];
    if (serie == '\n') {
      resultat = compteur;

    }
    compteur++;
  }
  return (resultat);
}

//-------------------------------------------------------   handleSubmit ------------------------------------------------------------------------------------------------------------------------------

void handleSubmit()
{

//Serial.print(server.args());
  if (server.args() == 3) {

    String text_num_volet = server.arg(1);
    VOLET_CHOISI = text_num_volet.toInt();
    if (VOLET_CHOISI > NB_VOLETS_DECLARES ){  // 0 = tous , 1 = premier volet
      VOLET_CHOISI=NB_VOLETS_DECLARES;
    }
    String text_sens_volet = server.arg(0);
    SENS_CHOISI = text_sens_volet.toInt();

    if  (VOLET_CHOISI == 0 )  {
        construire_line(1);
        server.send(200, "text/html", " Selection de tous les volets " + PAGE_ACCEUIL);
        // on deplace tous les volets
        traite_tous_les_volets(SENS_CHOISI);
    }     
    else{
          VOLET_NUMBER = VOLET_CHOISI-1;
          lire_volet(VOLET_NUMBER,true);;
          construire_line(1) ;
          server.send(200, "text/html",PAGE_ACCEUIL);
          if (SENS_CHOISI == 0 ) {
            descendre_volet(VOLET_NUMBER);
          }
          else
          {
            monter_volet(VOLET_NUMBER);
          }
          
    }      
  }
}
//-------------------------------------------------------  monter_volet  ------------------------------------------------------------------------------------------------------------------------------

void monter_volet(int numero_de_volet) {

  Serial.print("------------------------> Monter volet : "); Serial.println(numero_de_volet);
  VOLET_NUMBER = numero_de_volet;
  lire_volet(numero_de_volet, true);

  if (TYPE_VOLET == 0 || TYPE_VOLET == 2 ) { // volets Somfy
    if (SENS_TELECOMMANDE == 0) {
      BuildFrame(frame, HAUT);
    }
    else {
      BuildFrame(frame, BAS);
    }
    send_tram_to_volet(frame);

    if (TYPE_VOLET == 2) {  // volet somfy avec un stop au bout de DELAY_TO_STOP_FOR_TYPE_2 millisecondes
      delay (DELAY_TO_STOP_FOR_TYPE_2);
      stop_volet(numero_de_volet);
      // un autre des fois que le premier n'aurait pas marché
      delay (1000);
      stop_volet(numero_de_volet);
    } 
  }
  else if (TYPE_VOLET == 1)     // volets Bubendorff
  {
    if ( PIN_BUBENDORF_DEFINED == false) {
      PIN_BUBENDORF_DEFINED = true;
      if ( SENS_TELECOMMANDE == 0) {
        PIN_DESCENTE = ROLLING_CODE;
        PIN_MONTE = ADRESSE_TELECOMMANDE;
      }
      else {
        PIN_DESCENTE = ADRESSE_TELECOMMANDE;
        PIN_MONTE = ROLLING_CODE;
      }
      pinMode(PIN_DESCENTE, OUTPUT);
      pinMode(PIN_MONTE, OUTPUT);
      digitalWrite(PIN_DESCENTE, LOW);
      digitalWrite(PIN_MONTE, LOW);
    }
    digitalWrite(PIN_MONTE, HIGH);
    delay (1000);
    digitalWrite(PIN_MONTE, LOW);
    lire_volet(VOLET_NUMBER, false);
  }
}
//-------------------------------------------------------  descendre_volet  ------------------------------------------------------------------------------------------------------------------------------

void descendre_volet(int numero_de_volet) {

  Serial.print("------------------------> Descendre volet : "); Serial.println(numero_de_volet);
  VOLET_NUMBER = numero_de_volet;
  lire_volet(numero_de_volet, true);

  if (TYPE_VOLET == 0 || TYPE_VOLET == 2) {  // volets Somfy

    if (SENS_TELECOMMANDE == 0) {
      BuildFrame(frame, BAS);
    }
    else {
      BuildFrame(frame, HAUT);
    }

    send_tram_to_volet(frame);
    
    if (TYPE_VOLET == 2) {  // volet somfy avec un stop au bout de DELAY_TO_STOP_FOR_TYPE_2 millisecondes
      delay (DELAY_TO_STOP_FOR_TYPE_2);
      stop_volet(numero_de_volet);
      // un autre des fois que le premier n'aurait pas marché
      delay (1000);
      stop_volet(numero_de_volet);
    } 
  }
  else if (TYPE_VOLET == 1)      // volets Bubendorff
  {
    if ( PIN_BUBENDORF_DEFINED == false) {
      PIN_BUBENDORF_DEFINED = true;
      if ( SENS_TELECOMMANDE == 0) {
        PIN_DESCENTE = ROLLING_CODE;
        PIN_MONTE = ADRESSE_TELECOMMANDE;
      }
      else {
        PIN_DESCENTE = ADRESSE_TELECOMMANDE;
        PIN_MONTE = ROLLING_CODE;
      }
      pinMode(PIN_DESCENTE, OUTPUT);
      pinMode(PIN_MONTE, OUTPUT);
      digitalWrite(PIN_DESCENTE, LOW);
      digitalWrite(PIN_MONTE, LOW);
    }
    digitalWrite(PIN_DESCENTE, HIGH);
    delay (1000);
    digitalWrite(PIN_DESCENTE, LOW);
    lire_volet(VOLET_NUMBER, false);
  }
}
//-------------------------------------------------------  stop_volet  ------------------------------------------------------------------------------------------------------------------------------

void stop_volet(int VOLET_NUMBER) {

 if (TYPE_VOLET == 0 || TYPE_VOLET == 2) {  // pas de stop pour Bubendorff 
  

    lire_volet(VOLET_NUMBER,true);
    Serial.println("Stop");
    BuildFrame(frame, STOP);

    send_tram_to_volet(frame);
  }
}
void programme_volet(int VOLET_NUMBER) {

 if (TYPE_VOLET == 0 || TYPE_VOLET == 2) {  // pas de programmation pour Bubendorff 
    lire_volet( VOLET_NUMBER,true);
    Serial.println("Prog");
    BuildFrame(frame, PROG);

    send_tram_to_volet(frame);
 }
}
//-------------------------------------------------------  lire_volet  ------------------------------------------------------------------------------------------------------------------------------
void lire_volet( int VOLET_NUMBER, boolean silent ) {

  // VOLET_NUMBER = numero du volet
  // silent       = true si on veut pas le resultat sur la ligne serie, false si on le veut

  // 28 bytes voir organisation dans initialize_volet

  EEPROM.get( SIZE_REC_VOLET * VOLET_NUMBER, ROLLING_CODE);
  EEPROM.get(2 +  SIZE_REC_VOLET * VOLET_NUMBER, ADRESSE_TELECOMMANDE);
  EEPROM.get(6 +  SIZE_REC_VOLET * VOLET_NUMBER, SENS_TELECOMMANDE);
  EEPROM.get(7 +  SIZE_REC_VOLET * VOLET_NUMBER, TYPE_VOLET);
  EEPROM.get(8 +  SIZE_REC_VOLET * VOLET_NUMBER, VALIDATION_PROG_AUTO);
// libre 1 byte  EEPROM.get(9 
  EEPROM.get(10 +  SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSHINE);
  EEPROM.get(12 +  SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSET);
  NOM_DU_VOLET = "              ";
  //EEPROM.get(14 +  SIZE_REC_VOLET * VOLET_NUMBER, NOM_DU_VOLET);
  read_nom_volet_eeprom();

  if (silent == false ) {
    affiche_volet(VOLET_NUMBER);

  }
}
//-------------------------------------------------------  initialize_volet  ------------------------------------------------------------------------------------------------------------------------------

void initialize_volet( int VOLET_NUMBER) {

  // 288 bytes sont reservés pour chaque volet
  //  0 = ROLLING_CODE          unsigned short rolling code              (16 bits)  (2 byte)
  //  2 = ADRESSE_TELECOMMANDE unsigned int adresse de la telecommande  (32 bits)  (4 byte)
  //  6 = SENS_TELECOMMANDE   byte 0= normal ou 1=inverse sens du volet (8 bits)  (1 byte)
  //  7 = TYPE_VOLET          byte 0= somfy ou 1=BUBENDORFF , 2 = Somfy + stop apres delai de DELAY_TO_STOP_FOR_TYPE_2 millisecondes         (8 bits)  (1 byte)
  //  8 = VALIDATION_PROG_AUTO byte 0= automatique non accepté 1=full auto accepté,    2=descente auto acceptée seulement , 3 = montée automatique acceptée seuelement (8 bits)  (1 byte)
  //  9 = libre 1 byte
  //  10 = OFFSET_MN_SUNSHINE  unsigned short offset_mn_sunshine  = nombre de minute en + ou en - apres le lever du soleil pour lever ce volet ( 2 bytes)
  //  12 = OFFSET_MN_SUNSET  unsigned short offset_mn_sunset  = nombre de minute en + ou en - apres le coucher du soleil pour baisser ce volet ( 2 bytes)
  char serie_2;

  lire_volet(VOLET_NUMBER, false);


  Serial.print  ("Etes vous certain de vouoir initialiser ce volet o/n  : ");

  boolean continue_lecture = true;
  boolean reponse;

  reponse = get_oui_non_serie(false);

  if ( reponse == true ) {
    continue_lecture = false;
    Serial.println(" ");
    Serial.print     ("Type du volet 0 = SOMFY, 1 = BUBENDORFF, 2 = Somfy + Stop: ");
    TYPE_VOLET = get_byte_serie (false);
    if (TYPE_VOLET == 0 || TYPE_VOLET == 2 ) {
      Serial.print   ("Donnez l'adresse du volet                                : ");
      ADRESSE_TELECOMMANDE = get_unsigned_int_serie (false);
      Serial.print   ("Donnez le Rolling code du volet                          : ");
      newRollingCode = get_unsigned_short_serie (false);
    }
    else if (TYPE_VOLET == 1 ) {
      Serial.print   ("Donnez le numero de pin arduino pour monter les volets   : ");
      ADRESSE_TELECOMMANDE = get_unsigned_int_serie (false);
      Serial.print   ("Donnez le numero de pin arduino pour descendre les volets: ");
      newRollingCode = get_unsigned_short_serie (false);
    }
    Serial.print     ("Voulez vous inverser le sens du volet o/n                : ");
    reponse = get_oui_non_serie(true);
    if ( reponse == true) {
      SENS_TELECOMMANDE = 1;
      Serial.println("on inverse");
    }
    else {
      SENS_TELECOMMANDE = 0;
      Serial.println("on n'inverse pas");
    }
    Serial.println ("Code d'acceptation de programation");
    Serial.println ("0 activation automatique interdite");
    Serial.println ("1 activation automatique completement autorisée");
    Serial.println ("2 activation automatique autorisée en descente uniquement");
    Serial.print   ("3 activation automatique autorisée en montée uniquement  : ");
    VALIDATION_PROG_AUTO = get_byte_serie (false);


    Serial.print   ("Donnez xxx en plus ou en moins --> Monter le volet a xxx minutes avant (-) ou apres (+) le lever du soleil      : ");
    OFFSET_MN_SUNSHINE =   get_signed_short_serie (false);
    
    Serial.print   ("Donnez xxx en plus ou en moins --> Descendre le volet a xxx minutes avant (-) ou apres (+) le coucher du soleil : ");
    OFFSET_MN_SUNSET =   get_signed_short_serie (false);

    Serial.print   ("Donnez le nom du volet 12 caracteres maximum                                                                    : ");
    boolean boucle_de_saisie = true;
    while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      NOM_DU_VOLET = Serial.readString(); // read the incoming byte:
        boucle_de_saisie = false;
      }
  }
  
    check_nom_volet();

    Serial.println(" ");
    Serial.println(" ");
    Serial.print  ("Nouvelles valeurs du volet     : ");  Serial.println(VOLET_NUMBER) ;
    ROLLING_CODE = newRollingCode;

    affiche_volet(VOLET_NUMBER);
    // on peut ecrire le volet
    EEPROM.put( SIZE_REC_VOLET * VOLET_NUMBER, newRollingCode);
    EEPROM.put(2  + SIZE_REC_VOLET * VOLET_NUMBER, ADRESSE_TELECOMMANDE);
    EEPROM.put(6  + SIZE_REC_VOLET * VOLET_NUMBER, SENS_TELECOMMANDE);
    EEPROM.put(7  + SIZE_REC_VOLET * VOLET_NUMBER, TYPE_VOLET);
    EEPROM.put(8  + SIZE_REC_VOLET * VOLET_NUMBER, VALIDATION_PROG_AUTO);
//    EEPROM.put(9 1 byte libre 
    EEPROM.put(10  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSHINE);
    EEPROM.put(12  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSET);
    //EEPROM.put(14  + SIZE_REC_VOLET * VOLET_NUMBER, NOM_DU_VOLET);
    write_nom_volet_eeprom();
    EEPROM.commit();
  }
}
//-------------------------------------------------------  check_nom_volet  ------------------------------------------------------------------------------------------------------------------------------
void check_nom_volet()
{
  int nbr_car = NOM_DU_VOLET.length();
  int val_ascii;
  //Serial.println("");
  //Serial.print(" Check : ");Serial.print(nbr_car);Serial.print(" : ");Serial.println(NOM_DU_VOLET);
  int nb_car_non_nul=0;
  int carriarge_return=0;
  for  (int compteur = 0  ; compteur <= nbr_car ; compteur++)
  {
    val_ascii = NOM_DU_VOLET[compteur];
    if (val_ascii != 0 )
    {
      nb_car_non_nul++;
    }
    else
    {
      // on est sur un nul, on sort de la boucle
      break;
    }
    if (val_ascii == 10 )
    {
      carriarge_return = nb_car_non_nul;
    }

    //Serial.print(NOM_DU_VOLET[compteur]);Serial.print("  : " );Serial.println(val_ascii);
  }
  if (carriarge_return > 0 )
    {
        // Acquisition serie
        if ( carriarge_return <=12 )
        {
          // c'est tout bon on fait rien
        }
        else
        {
          // trop long on tronque
          String nom_temp = NOM_DU_VOLET;
          NOM_DU_VOLET = "              ";
          for (int compteur =0; compteur < 12 ; compteur ++ )
          {
            NOM_DU_VOLET[compteur] = nom_temp[compteur];
          }
          NOM_DU_VOLET[12] ='\r';
          NOM_DU_VOLET[13] ='\n';
        }
    }
    else  // Acquisition par le web
    {
      if (nb_car_non_nul > 12 )
      {
        nb_car_non_nul = 12;
      }
        NOM_DU_VOLET[nb_car_non_nul] ='\r';
        NOM_DU_VOLET[nb_car_non_nul+1] ='\n';
    }
     //  Serial.print("  nb car non nul : ");Serial.print(nb_car_non_nul);  Serial.print("  position retour chariot :  " );Serial.println(carriarge_return);
}
//-------------------------------------------------------  get_oui_non_serie  ------------------------------------------------------------------------------------------------------------------------------

boolean get_oui_non_serie(boolean silent) {

  // retour boolean reponse = true pour oui = false pour non

  boolean  boucle_de_saisie = true;
  boolean  reponse = true;
  String read_serie = "               ";


  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {

      if (Serial.available() > 0) {
        read_serie = "               ";
        read_serie = Serial.readString();
        char characunique = read_serie[0];
        int nb_car = compte_characteres(read_serie);
        if (nb_car == 1 ) {
          if ( characunique == 'o' || characunique == 'y') {
            boucle_de_saisie = false;
            reponse = true;
            if (silent == false) {
              Serial.println('o');
            }
          }
          else if ( characunique == 'n' ) {
            boucle_de_saisie = false;
            reponse = false;
            if (silent == false) {
              Serial.println('n');
            }
          }
          else {
            Serial.println("Reponse incorrecte o, n, ou y");
            boucle_de_saisie = true;
          }
        }
        else
        {
          Serial.println("Reponse incorrecte o, n, ou y");
          boucle_de_saisie = true;
        }
        
      }
    }
  }
  return (reponse);
}

//-------------------------------------------------------  get_unsigned_int_serie  ------------------------------------------------------------------------------------------------------------------------------

unsigned int  get_unsigned_int_serie (boolean silent) {

  String read_serie = "               ";
  unsigned int retour_int;
  boolean boucle_de_saisie = true;
  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      read_serie = "               ";
      read_serie = Serial.readString();
      retour_int = read_serie.toInt();
      if (silent == false) {
        Serial.println(retour_int);
      }
      boucle_de_saisie = false;
    }
  }
  return (retour_int);
}

//-------------------------------------------------------  get_byte_serie  ------------------------------------------------------------------------------------------------------------------------------

byte  get_byte_serie (boolean silent) {

  String read_serie = "               ";
  byte retour_byte;
  boolean boucle_de_saisie = true;
  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      read_serie = "               ";
      read_serie = Serial.readString();
      retour_byte = read_serie.toInt();
      if (silent == false) {
        Serial.println(retour_byte);
      }
      boucle_de_saisie = false;
    }
  }
  return (retour_byte);
}
//-------------------------------------------------------  get_unsigned_short_serie  ------------------------------------------------------------------------------------------------------------------------------

unsigned short  get_unsigned_short_serie (boolean silent) {

  String read_serie = "               ";
  unsigned short retour_short;
  boolean boucle_de_saisie = true;
  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      read_serie = "               ";
      read_serie = Serial.readString();
      retour_short = read_serie.toInt();
      if (silent == false) {
        Serial.println(retour_short);
      }
      boucle_de_saisie = false;
    }
  }
  return (retour_short);
}

//-------------------------------------------------------  get_short_serie  ------------------------------------------------------------------------------------------------------------------------------

short  get_short_serie (boolean silent) {

  String read_serie = "               ";
  short retour_short;
  boolean boucle_de_saisie = true;
  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      read_serie = "               ";
      read_serie = Serial.readString();
      retour_short = read_serie.toInt();
      if (silent == false) {
        Serial.println(retour_short);
      }
      boucle_de_saisie = false;
    }
  }
  return (retour_short);
}

//-------------------------------------------------------  get_signed_short_serie  ------------------------------------------------------------------------------------------------------------------------------


short  get_signed_short_serie (boolean silent) {

  String read_serie = "               ";
  short retour_short;
  boolean boucle_de_saisie = true;
  while (boucle_de_saisie == true ) {
    if (Serial.available() > 0) {
      read_serie = "               ";
      read_serie = Serial.readString();
      retour_short = read_serie.toInt();
      if (silent == false) {
        Serial.println(retour_short);
      }
      boucle_de_saisie = false;
    }
  }
  return (retour_short);
}

//-------------------------------------------------------  affiche_volet  ------------------------------------------------------------------------------------------------------------------------------

void affiche_volet( int VOLET_NUMBER) {
  Serial.println(" ");

  Serial.print    ("Numero du volet              : ");  Serial.println(VOLET_NUMBER);
  Serial.print    ("Nom du volet                 : ");  Serial.print(NOM_DU_VOLET); 
  if (TYPE_VOLET == 0 || TYPE_VOLET == 2) {
    Serial.print  ("Rolling code pour le volet   : ");  Serial.println(ROLLING_CODE);
    Serial.print  ("Adresse pour le volet        : ");  Serial.print(ADRESSE_TELECOMMANDE); Serial.print(" soit en Hexa " ); Serial.println(ADRESSE_TELECOMMANDE, HEX );
  }
  else
  {
    Serial.print  ("Pin de montée du volet       : ");  Serial.println(ADRESSE_TELECOMMANDE);
    Serial.print  ("Pin de descente du volet     : ");  Serial.println(ROLLING_CODE);
  }

  String text_type_volet = "            ";
  if (TYPE_VOLET == 0 ) {
    text_type_volet =      "Somfy       ";
  }
  else if ( TYPE_VOLET == 1) {
    text_type_volet =      "Bubendorff  ";
  }
   else if  ( TYPE_VOLET == 2) {
    text_type_volet =      "Somfy + stop  ";
  }
  Serial.print  ("Type de volet                : ");  Serial.println(text_type_volet);

  String text_sens_telecommande = "       ";
  if (SENS_TELECOMMANDE == 0 ) {
    text_sens_telecommande =      "Normal ";
  }
  else {
    text_sens_telecommande =      "Inverse";
  }
  Serial.print  ("Sens pour du volet           : ");  Serial.println(text_sens_telecommande);

  String txt_validation_auto;
  switch (VALIDATION_PROG_AUTO)  {
    case 0:
      txt_validation_auto = "Pas d'automatisation autorisée";
      break;
    case 1:
      txt_validation_auto = "Automatisation complète autorisée";
      break;
    case 2:
      txt_validation_auto = "Descente automatique autorisée uniquement";
      break;
    case 3:
      txt_validation_auto = "Montée automatique autorisée uniquement";
      break;
    default:
      txt_validation_auto = "Erreur de code : ";
      break;
  }// end case
  
  Serial.print  ("Automatisation autorisee     : ");  Serial.println(txt_validation_auto);

  Serial.print  ("Nbr minutes +/- lever soleil : ");  Serial.println(OFFSET_MN_SUNSHINE);
  Serial.print  ("Nbr minutes +/- coucher solei: ");  Serial.println(OFFSET_MN_SUNSET); 

}

//-------------------------------------------------------  BuildFrame  ------------------------------------------------------------------------------------------------------------------------------

void BuildFrame(byte *frame, byte button) {
  unsigned short code;
  EEPROM.get( SIZE_REC_VOLET * VOLET_NUMBER, code);

  frame[0] = 0xA7; // Encryption key. Doesn't matter much
  frame[1] = button << 4;  // Which button did  you press? The 4 LSB will be the checksum
  frame[2] = code >> 8;    // Rolling code (big endian)
  frame[3] = code;         // Rolling code
  frame[4] = ADRESSE_TELECOMMANDE >> 16; // Remote address
  frame[5] = ADRESSE_TELECOMMANDE >>  8; // Remote address
  frame[6] = ADRESSE_TELECOMMANDE;       // Remote address

  Serial.print("Frame         : ");
  for (byte i = 0; i < 7; i++) {
    if (frame[i] >> 4 == 0) { //  Displays leading zero in case the most significant
      Serial.print("0");     // nibble is a 0.
    }
    Serial.print(frame[i], HEX); Serial.print(" ");
  }

  // Checksum calculation: a XOR of all the nibbles
  checksum = 0;
  for (byte i = 0; i < 7; i++) {
    checksum = checksum ^ frame[i] ^ (frame[i] >> 4);
  }
  checksum &= 0b1111; // We keep the last 4 bits only


  //Checksum integration
  frame[1] |= checksum; //  If a XOR of all the nibbles is equal to 0, the blinds will
  // consider the checksum ok.

  Serial.println(""); Serial.print("With checksum : ");
  for (byte i = 0; i < 7; i++) {
    if (frame[i] >> 4 == 0) {
      Serial.print("0");
    }
    Serial.print(frame[i], HEX); Serial.print(" ");
  }
  // Obfuscation: a XOR of all the bytes
  for (byte i = 1; i < 7; i++) {
    frame[i] ^= frame[i - 1];
  }

  Serial.println(""); Serial.print("Obfuscated    : ");
  for (byte i = 0; i < 7; i++) {
    if (frame[i] >> 4 == 0) {
      Serial.print("0");
    }
    Serial.print(frame[i], HEX); Serial.print(" ");
  }
  Serial.println("");
  code ++;   // incremente le romming code

  EEPROM.put( SIZE_REC_VOLET * VOLET_NUMBER, code); //  We store the value of the rolling code in the
  // EEPROM. It should take up to 2 adresses but the
  // Arduino function takes care of it.
  EEPROM.commit();
  lire_volet(VOLET_NUMBER, false);
}
//-------------------------------------------------------  SendCommand  ------------------------------------------------------------------------------------------------------------------------------

void SendCommand(byte *frame, byte sync) {
  if (sync == 2) { // Only with the first frame.
    //Wake-up pulse & Silence
    // PORTD |= 1<<PORT_TX;
    digitalWrite(PORT_TX, HIGH);
    delayMicroseconds(9415);
    //PORTD &= !(1<<PORT_TX);
    digitalWrite(PORT_TX, LOW);
    delayMicroseconds(89565);
  }

  // Hardware sync: two sync for the first frame, seven for the following ones.
  for (int i = 0; i < sync; i++) {
    //PORTD |= 1<<PORT_TX;
    digitalWrite(PORT_TX, HIGH);
    delayMicroseconds(4 * SYMBOL);
    //PORTD &= !(1<<PORT_TX);
    digitalWrite(PORT_TX, LOW);
    delayMicroseconds(4 * SYMBOL);
  }

  // Software sync
  //PORTD |= 1<<PORT_TX;
  digitalWrite(PORT_TX, HIGH);
  delayMicroseconds(4550);
  //PORTD &= !(1<<PORT_TX);
  digitalWrite(PORT_TX, LOW);
  delayMicroseconds(SYMBOL);


  //Data: bits are sent one by one, starting with the MSB.
  for (byte i = 0; i < 56; i++) {
    if (((frame[i / 8] >> (7 - (i % 8))) & 1) == 1) {
      //PORTD &= !(1<<PORT_TX);
      digitalWrite(PORT_TX, LOW);
      delayMicroseconds(SYMBOL);
      //PORTD ^= 1<<5;
      digitalWrite(PORT_TX, HIGH);
      delayMicroseconds(SYMBOL);
    }
    else {
      //PORTD |= (1<<PORT_TX);
      digitalWrite(PORT_TX, HIGH);
      delayMicroseconds(SYMBOL);
      //PORTD ^= 1<<5;
      digitalWrite(PORT_TX, LOW);
      delayMicroseconds(SYMBOL);
    }
  }

  //PORTD &= !(1<<PORT_TX);
  digitalWrite(PORT_TX, LOW);
  delayMicroseconds(30415); // Inter-frame silence
}

//-------------------------------------------------------  get_set_gen_param  ------------------------------------------------------------------------------------------------------------------------------

boolean get_set_gen_param(char get_set) {

  boolean error_proc = false;

  if (get_set == 'l') {
    // lecture des 2 paramètres généraux
    EEPROM.get(502,HEURE_MINI_DESCENTE_VOLETS);
    EEPROM.get(504,HEURE_MAXI_DESCENTE_VOLETS);
    EEPROM.get(506,HEURE_MINI_MONTE_VOLETS);
    EEPROM.get(508,HEURE_MAXI_MONTE_VOLETS);
    EEPROM.get(510, NB_VOLETS_DECLARES);
    EEPROM.get(499, OFFSET_VERSUS_SOLEIL);
    TIMEZONE=-OFFSET_VERSUS_SOLEIL ; // 2 en été, 1 en hiver
    EEPROM.get(501, MODE_AUTOMATIQUE);
    char time_mini_descente[6];
    Dusk2Dawn::min2str(time_mini_descente, HEURE_MINI_DESCENTE_VOLETS);
    char time_maxi_descente[6];
    Dusk2Dawn::min2str(time_maxi_descente, HEURE_MAXI_DESCENTE_VOLETS);
    char time_mini_monte[6];
    Dusk2Dawn::min2str(time_mini_monte, HEURE_MINI_MONTE_VOLETS);
    char time_maxi_monte[6];
    Dusk2Dawn::min2str(time_maxi_monte, HEURE_MAXI_MONTE_VOLETS);
  
    Serial.println("");
    if (MODE_AUTOMATIQUE == true ) {
      Serial.println("Mode                                                  : Automatique"); 
    }
    else
    {
      Serial.println("Mode                                                  : Manuel"); 
    }
    Serial.print("Nombre de volets déclarés                             : "); Serial.println(NB_VOLETS_DECLARES);
    Serial.print("Offset / soleil 1 en hiver, 2 en été                  : "); Serial.println(OFFSET_VERSUS_SOLEIL);
    Serial.print("heure mini monte volets (nbre minutes depuis 00:00)   : "); Serial.print(HEURE_MINI_MONTE_VOLETS); Serial.print( " ==> ("); Serial.print(time_mini_monte); Serial.println( ")");
    Serial.print("heure maxi monte volets (nbre minutes depuis 00:00)   : "); Serial.print(HEURE_MAXI_MONTE_VOLETS); Serial.print( " ==> ("); Serial.print(time_maxi_monte); Serial.println( ")");
    Serial.print("heure mini descente volets (nbre minutes depuis 00:00): "); Serial.print(HEURE_MINI_DESCENTE_VOLETS); Serial.print( " ==> ("); Serial.print(time_mini_descente); Serial.println( ")");
    Serial.print("heure maxi descente volets (nbre minutes depuis 00:00): "); Serial.print(HEURE_MAXI_DESCENTE_VOLETS); Serial.print( " ==> ("); Serial.print(time_maxi_descente); Serial.println( ")");
  }
  else if (get_set == 'e') {
    Serial.println("");
    Serial.print     ("Donnez le nombre de volets entre 0 et 15                      : ");
    NB_VOLETS_DECLARES = get_byte_serie (false);
    error_proc = false;
    if (NB_VOLETS_DECLARES < 0 || NB_VOLETS_DECLARES > 15 ) {
      error_proc = true;
      Serial.println("");
      Serial.println(" --------------> Attention le nombre de volets doit etre compris entre 0 et 15 < ------------------------");
      NB_VOLETS_DECLARES = 0;
    }
    if (error_proc == false) {
      Serial.print("Offset par rapport au soleil (En france -1 en hiver, -2 en été: ");
      OFFSET_VERSUS_SOLEIL = get_short_serie (false);
      TIMEZONE=-OFFSET_VERSUS_SOLEIL ; // 2 en été, 1 en hiver
      get_ntp_time();

      Serial.print("heure mini monte volets (nbre minutes depuis 00:00)           : ");
      HEURE_MINI_MONTE_VOLETS = get_unsigned_short_serie  (true);
      char time_mini_monte[6];
      Dusk2Dawn::min2str(time_mini_monte, HEURE_MINI_MONTE_VOLETS);
      Serial.print (HEURE_MINI_MONTE_VOLETS); Serial.print ("   (");Serial.print (time_mini_monte);Serial.println (")");

      Serial.print("heure maxi monte volets (nbre minutes depuis 00:00)           : ");
      HEURE_MAXI_MONTE_VOLETS = get_unsigned_short_serie  (true);
      char time_maxi_monte[6];
      Dusk2Dawn::min2str(time_maxi_monte, HEURE_MAXI_MONTE_VOLETS);
      Serial.print (HEURE_MAXI_MONTE_VOLETS); Serial.print ("   (");Serial.print (time_maxi_monte);Serial.println (")");

      Serial.print("heure mini descente volets (nbre minutes depuis 00:00)        : ");
      HEURE_MINI_DESCENTE_VOLETS = get_unsigned_short_serie  (true);
      char time_mini_descente[6];
      Dusk2Dawn::min2str(time_mini_descente, HEURE_MINI_DESCENTE_VOLETS);
      Serial.print (HEURE_MINI_DESCENTE_VOLETS); Serial.print ("   (");Serial.print (time_mini_descente);Serial.println (")");
     
      Serial.print("heure maxi descente volets (nbre minutes depuis 00:00)        : ");
      HEURE_MAXI_DESCENTE_VOLETS = get_unsigned_short_serie  (true);
      char time_maxi_descente[6];
      Dusk2Dawn::min2str(time_maxi_descente, HEURE_MAXI_DESCENTE_VOLETS);
      Serial.print (HEURE_MAXI_DESCENTE_VOLETS); Serial.print ("   (");Serial.print (time_maxi_descente);Serial.println (")");

      EEPROM.put(502,HEURE_MINI_DESCENTE_VOLETS);
      EEPROM.put(504,HEURE_MAXI_DESCENTE_VOLETS);      
      EEPROM.put(506,HEURE_MINI_MONTE_VOLETS);
      EEPROM.put(508,HEURE_MAXI_MONTE_VOLETS);
      EEPROM.put(510, NB_VOLETS_DECLARES);
      EEPROM.put(499, OFFSET_VERSUS_SOLEIL);
      TIMEZONE=-OFFSET_VERSUS_SOLEIL ; // 2 en été, 1 en hiver
      EEPROM.put(501, MODE_AUTOMATIQUE);
      EEPROM.commit();
    }
      Serial.println("");
  }
  else if (get_set == 'x' ){  // initialisation de l'EEPROM  

// partie générale
      HEURE_MINI_DESCENTE_VOLETS=0;
      EEPROM.put(502,HEURE_MINI_DESCENTE_VOLETS);
      HEURE_MAXI_DESCENTE_VOLETS=0;
      EEPROM.put(504,HEURE_MAXI_DESCENTE_VOLETS);
      HEURE_MINI_MONTE_VOLETS=0;
      EEPROM.put(506,HEURE_MINI_MONTE_VOLETS);
      HEURE_MAXI_MONTE_VOLETS=0;
      EEPROM.put(508,HEURE_MAXI_MONTE_VOLETS);
      NB_VOLETS_DECLARES=0;
      EEPROM.put(510, NB_VOLETS_DECLARES);
      OFFSET_VERSUS_SOLEIL=0;
      EEPROM.put(499, OFFSET_VERSUS_SOLEIL);
    for (int compteur = 0 ; compteur < 16 ; compteur ++ ) {
        newRollingCode = 0;
        EEPROM.put( SIZE_REC_VOLET * VOLET_NUMBER, newRollingCode);
        ADRESSE_TELECOMMANDE=0;
        EEPROM.put(2  + SIZE_REC_VOLET * VOLET_NUMBER, ADRESSE_TELECOMMANDE);
        SENS_TELECOMMANDE=0;
        EEPROM.put(6  + SIZE_REC_VOLET * VOLET_NUMBER, SENS_TELECOMMANDE);
        TYPE_VOLET=0;
        EEPROM.put(7  + SIZE_REC_VOLET * VOLET_NUMBER, TYPE_VOLET);
        VALIDATION_PROG_AUTO=0;
        EEPROM.put(8  + SIZE_REC_VOLET * VOLET_NUMBER, VALIDATION_PROG_AUTO);
//        EEPROM.put(9  1 byte libre
        OFFSET_MN_SUNSHINE = 0;
        EEPROM.put(10  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSHINE);
        OFFSET_MN_SUNSET = 0;
        EEPROM.put(12  + SIZE_REC_VOLET * VOLET_NUMBER, OFFSET_MN_SUNSET);
        NOM_DU_VOLET = "             " + String("\n");
        write_nom_volet_eeprom ();
    }
    Serial.println (" ");
    Serial.println (" ");
    Serial.println ("Initialisation de l'EEPROM terminée");
    Serial.println (" ");
    Serial.println (" ");
  }
  else {
    error_proc = true;
  }
  return (error_proc);
}


//-------------------------------------------------------  write_nom_volet_eeprom  ------------------------------------------------------------------------------------------------------------------------------


void write_nom_volet_eeprom (){
  for ( int compteur = 0 ; compteur < 14; compteur ++) {
    EEPROM.put(compteur + 14  + SIZE_REC_VOLET * VOLET_NUMBER, NOM_DU_VOLET[compteur]);
  }
}

//-------------------------------------------------------  read_nom_volet_eeprom  ------------------------------------------------------------------------------------------------------------------------------

void read_nom_volet_eeprom (){
  for ( int compteur = 0 ; compteur < 14; compteur ++) {
    EEPROM.get(compteur + 14  + SIZE_REC_VOLET * VOLET_NUMBER, NOM_DU_VOLET[compteur]);
  }
}


//-------------------------------------------------------  get_ntp_time  ------------------------------------------------------------------------------------------------------------------------------


void get_ntp_time(){
Udp.begin(localPort);
    Serial.print("Local port: ");
  Serial.println(Udp.localPort());
  Serial.println("waiting for sync");
  setSyncProvider(getNtpTime);
  setSyncInterval(86400); // sync une fois par jour
  //digitalClockDisplay();
  if (timeStatus() != timeNotSet) {
    if (now() != prevDisplay) { //update the display only if time has changed
      prevDisplay = now();
      digitalClockDisplay();
    }
  }
}


//-------------------------------------------------------   NTP code  ------------------------------------------------------------------------------------------------------------------------------


const int NTP_PACKET_SIZE = 48; // NTP time is in the first 48 bytes of message
byte packetBuffer[NTP_PACKET_SIZE]; //buffer to hold incoming & outgoing packets

time_t getNtpTime()
{
  IPAddress ntpServerIP; // NTP server's ip address

  while (Udp.parsePacket() > 0) ; // discard any previously received packets
  Serial.println("Transmit NTP Request");
  // get a random server from the pool
  WiFi.hostByName(ntpServerName, ntpServerIP);
  Serial.print(ntpServerName);
  Serial.print(": ");
  Serial.println(ntpServerIP);
  sendNTPpacket(ntpServerIP);
  uint32_t beginWait = millis();
  while (millis() - beginWait < 1500) {
    int size = Udp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      Serial.println("Receive NTP Response");
      Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
      unsigned long secsSince1900;
      // convert four bytes starting at location 40 to a long integer
      secsSince1900 =  (unsigned long)packetBuffer[40] << 24;
      secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
      secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
      secsSince1900 |= (unsigned long)packetBuffer[43];
      return secsSince1900 - 2208988800UL + TIMEZONE * SECS_PER_HOUR;
    }
  }
  Serial.println("No NTP Response :-(");
  return 0; // return 0 if unable to get the time
}

//-------------------------------------------------------   sendNTPpacket ------------------------------------------------------------------------------------------------------------------------------

// send an NTP request to the time server at the given address
void sendNTPpacket(IPAddress &address)
{
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}


//-------------------------------------------------------   digitalClockDisplay ------------------------------------------------------------------------------------------------------------------------------

void digitalClockDisplay()
{
  // digital clock display of the time
  Serial.print(hour());
  printDigits(minute());
  printDigits(second());
  Serial.print(" ");
  Serial.print(day());
  Serial.print(".");
  Serial.print(month());
  Serial.print(".");
  Serial.print(year());
  Serial.println();
  delay(10000);
}


//-------------------------------------------------------   printDigits ------------------------------------------------------------------------------------------------------------------------------


void printDigits(int digits)
{
  // utility for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if (digits < 10)
    Serial.print('0');
  Serial.print(digits);
}

//-------------------------------------------------------   get_Sunset ------------------------------------------------------------------------------------------------------------------------------

int get_Sunset(){

    int minute_sunset   = Clermont_Ferrand.sunset(year(), month(), day(), false) - OFFSET_VERSUS_SOLEIL* 60;
    return(minute_sunset);
}
//-------------------------------------------------------   get_Sunrise ------------------------------------------------------------------------------------------------------------------------------


int get_Sunrise(){
  
    int minute_Sunrise  = Clermont_Ferrand.sunrise(year(), month(), day(), false) - OFFSET_VERSUS_SOLEIL* 60;
    
    return(minute_Sunrise);
}

// ---------------------------------------------------   construire_line ------------------------------------------------------------------------------------------------------------------------------

void construire_line(int cas ) 

// cas = 0 ==> page de login
// cas = 1 ==> page principale
// cas = 2 ==> page configuration générale
// cas = 3 ==> page configuration volet
{

  String buffer_heure = String( hour());
  String buffer_minute= String( minute()); 
  String buffer_year= String( year()); 
  String buffer_month= String( month()); 
  String buffer_day= String( day()); 
  char time3[] = "00:00";
  Dusk2Dawn::min2str(time3, get_Sunrise());
  char time4[] = "00:00";
  int compteur;
  Dusk2Dawn::min2str(time4, get_Sunset());
  String str_checked;
  String Str_nb_volet;
  String Str_offset_soleil;
  String Str_mini_monte_vol;
  String Str_maxi_monte_vol;
  String Str_mini_descente_vol;
  String Str_maxi_descente_vol;
  String Str_type_volet ;
  String str_nom_typ;
  String Str_Rolling; 
  String Str_add_volet; 
  String Str_sens_commande; 
  String Str_automatisation; 
  String Str_offset_leve; 
  String Str_offset_descente; 
  unsigned int sav_volet_number;
  String Str_numero_volet;

  lire_volet(VOLET_NUMBER,true);
    
  //PAGE_ACCEUIL =  String("<style>.styled {border: 0;line-height: 2.5;padding:0 20px;font-size:2rem;text-align:center;color: #fff;text-shadow:1px 1px 1px #000;border-radius:10px;background-color:rgba(220, 0, 0, 1);background-image:linear-gradient(to top left,rgba(0, 0, 0, .2),rgba(0, 0, 0, .2) 30%,rgba(0, 0, 0, 0));box-shadow:inset 2px 2px 3px rgba(255, 255, 255, .6),inset -2px -2px 3px rgba(0, 0, 0, .6);}.styled:hover{background-color:rgba(255, 0, 0, 1);}.styled:active{box-shadow:inset -2px -2px 3px rgba(255, 255, 255, .6),inset 2px 2px 3px rgba(0, 0, 0, .6);}input, textarea {  font: 1em sans-serif;width: 150px; box-sizing: border-box;border: 1px solid #999;}</style>");
  PAGE_ACCEUIL =    String("<style>.styled {border: 0;line-height: 2.5;padding:0 20px;font-size:2rem;text-align:center;color: #fff;text-shadow:1px 1px 1px #000;border-radius:10px;background-color:rgba(220, 0, 0, 1);background-image:linear-gradient(to top left,rgba(0, 0, 0, .2),rgba(0, 0, 0, .2) 30%,rgba(0, 0, 0, 0));box-shadow:inset 2px 2px 3px rgba(255, 255, 255, .6),inset -2px -2px 3px rgba(0, 0, 0, .6);}.styled:hover{background-color:rgba(255, 0, 0, 1);}.styled:active{box-shadow:inset -2px -2px 3px rgba(255, 255, 255, .6),inset 2px 2px 3px rgba(0, 0, 0, .6);}</style>");

  if (cas == 0 )  
  {
    
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("<html><br> <br> <font face=Comic Sans MS color=#0000DD size=12><body><form action='/login' method='POST'>Donnez identifiant et mot de passe <br> <br> <br> Identification:<input class=\"favorite styled\" type='text' name='USERNAME' placeholder='ident'><br><br>Mot de passe:<input type='password' name='PASSWORD' placeholder='code' size=10 class=\"favorite styled\"><br><br><input class=\"favorite styled\" type='submit' name='SUBMIT' value='Go'></form><br></body></font></html>");
  }
  else if (cas == 1 )
  {
      if (MODE_AUTOMATIQUE == true) {
          PAGE_ACCEUIL = PAGE_ACCEUIL + String("Automatique  ");
      }
      else
      {
        PAGE_ACCEUIL = PAGE_ACCEUIL + String("Manuel  ");
      }
      PAGE_ACCEUIL = PAGE_ACCEUIL + String( buffer_year + "/" + buffer_month + "/" + buffer_day + " " + buffer_heure + ":" + buffer_minute +  " /\\ " + time3 +  " \\/ " + time4 );
      PAGE_ACCEUIL = PAGE_ACCEUIL +   String( "<!DOCTYPE HTML><html><head><meta name = \"viewport\" content = \"width = device-width, initial-scale = 1.0, maximum-scale = 1.0, user-scalable=0\"><title>Volets Cordeliers " + VERSION + "</title></head><body><h1>Volets Cordeliers " + VERSION + "</h1><a class=\"favorite styled\" href=\"Mode_automatique\"><button>Automatique</button></a>&nbsp;<a class=\"favorite styled\" href=\"Mode_manuel\"><button>Manuel</button></a>");

      if ( SENS_CHOISI == 0 ){
          PAGE_ACCEUIL = PAGE_ACCEUIL +  String("<FORM action=\"/\" method=\"post\"><P><h2><INPUT type=\"radio\" name=\"SENS\" value=\"1\"  >Monter<INPUT type=\"radio\" name=\"SENS\" value=\"0\" checked >Descente<BR><BR>");
      }
      else
      {
          PAGE_ACCEUIL = PAGE_ACCEUIL +  String("<FORM action=\"/\" method=\"post\"><P><h2><INPUT type=\"radio\" name=\"SENS\" value=\"1\" checked>Monter<INPUT type=\"radio\" name=\"SENS\" value=\"0\" >Descente<BR><BR>");
      }
      str_checked = "";
      if (VOLET_CHOISI == 0 ) {str_checked = "checked";}
      PAGE_ACCEUIL = PAGE_ACCEUIL +  String("<h3>Volets<br><INPUT type=\"radio\" name=\"VOLET\" value=\"0\" " + str_checked+ ">Tous<BR>");

      sav_volet_number = VOLET_NUMBER;
      for (compteur = 1 ; compteur <=  NB_VOLETS_DECLARES; compteur ++ )
      {
         VOLET_NUMBER=compteur-1;
         read_nom_volet_eeprom ();
         
         str_checked = "";
         if (VOLET_CHOISI == compteur ) {str_checked = "checked";}
         PAGE_ACCEUIL = PAGE_ACCEUIL +  String("<INPUT type=\"radio\" name=\"VOLET\" value=\"" + String(compteur)+ "\"" + str_checked +  ">" + NOM_DU_VOLET + "<BR>");
      }
      // restauration du numero de volet et du nom
      VOLET_NUMBER = sav_volet_number;
      read_nom_volet_eeprom ();

      PAGE_ACCEUIL = PAGE_ACCEUIL + String("<BR><INPUT class=\"favorite styled\" type=\"submit\" value=\"Go\" ></P></FORM></body></html><br><a href=\"cfg_gen\">config generale</a>___<a href=\"cfg_vol\">config volet</a>");
   }
   else if (cas == 2 )
   {
      Str_nb_volet = String(NB_VOLETS_DECLARES); 
      Str_offset_soleil = String(OFFSET_VERSUS_SOLEIL); 
      Str_mini_monte_vol = String(HEURE_MINI_MONTE_VOLETS); 
      Str_maxi_monte_vol = String(HEURE_MAXI_MONTE_VOLETS); 
      Str_mini_descente_vol = String(HEURE_MINI_DESCENTE_VOLETS); 
   
      Str_maxi_descente_vol = String(HEURE_MAXI_DESCENTE_VOLETS); 
      char time1[] = "00:00";
      Dusk2Dawn::min2str(time1, HEURE_MINI_MONTE_VOLETS);
      char time2[] = "00:00";
      Dusk2Dawn::min2str(time2, HEURE_MAXI_MONTE_VOLETS);
      char time3[] = "00:00";
      Dusk2Dawn::min2str(time3, HEURE_MINI_DESCENTE_VOLETS);
      char time4[] = "00:00";
      Dusk2Dawn::min2str(time4, HEURE_MAXI_DESCENTE_VOLETS);

      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("<!DOCTYPE html> <html><head><meta charset=utf-8 name = \"viewport\" content = \"width = device-width, initial-scale = 1.0, maximum-scale = 1.0, user-scalable=0\"><title>Configuration Générale</title></head><body><h1>Configuration Générale</h1>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <style>.styled {border: 0;line-height: 2.5;padding:0 10px;font-size:2rem;text-align:center;color: #fff;text-shadow:1px 1px 1px #000;border-radius:10px;background-color:rgba(220, 0, 0, 1);background-image:linear-gradient(to top left,rgba(0, 0, 0, .2),rgba(0, 0, 0, .2) 30%,rgba(0, 0, 0, 0));box-shadow:inset 2px 2px 3px rgba(255, 255, 255, .6),inset -2px -2px 3px rgba(0, 0, 0, .6);}.styled:hover{background-color:rgba(255, 0, 0, 1);}.styled:active{box-shadow:inset -2px -2px 3px rgba(255, 255, 255, .6),inset 2px 2px 3px rgba(0, 0, 0, .6);}</style> <html lang=\"en\"> <head> <title>Bootstrap Example</title></style></head>");
      PAGE_ACCEUIL =   PAGE_ACCEUIL + String("<form name=\"formulaire_cfg_gen\" action=\"formulaire_cfg_gen\" method=\"get\"><input  type=\"text\" name=\"nb_volets_declares\" value=\"" + Str_nb_volet + "\"> # volets déclarés");
      PAGE_ACCEUIL =   PAGE_ACCEUIL + String("<br><input  type=\"text\" name=\"offset_soleil\" value=\"" + Str_offset_soleil + "\"> Soleil -1 = hiver, -2 = été <br><input  type=\"text\" name=\"heure_mini_monte_volets\" value=\"" + Str_mini_monte_vol + "\">" + " H mini monte (" + time1[0] + time1[1] + time1[2] + time1[3] + time1[4]+ ")" );
      PAGE_ACCEUIL =   PAGE_ACCEUIL + String("<br><input  type=\"text\" name=\"heure_maxi_monte_volets\" value=\"" + Str_maxi_monte_vol + "\" />" + " H maxi monte (" + time2[0] + time2[1] + time2[2] + time2[3] + time2[4] + ")");
      PAGE_ACCEUIL =   PAGE_ACCEUIL + String("<br><input  type=\"text\" name=\"heure_mini_descente_volets\" value=\"" + Str_mini_descente_vol + "\"/>" + " H mini descente(" + time3[0] + time3[1] + time3[2] + time3[3] + time3[4] + ")");
      PAGE_ACCEUIL =   PAGE_ACCEUIL + String("<br><input  type=\"text\" name=\"heure_maxi_descente_volets\" value=\"" + Str_maxi_descente_vol + "\"/>" + " H maxi descente(" + time4[0] + time4[1] + time4[2] + time4[3] + time4[4] + ")" + "<br><br><br><input  class=\"favorite styled\"  type=\"submit\" value=\"Valider\"></form>");

   }
   else if (cas == 3 )
   {
      Str_numero_volet = String(VOLET_NUMBER);

      str_nom_typ ="(Somfy)";
      if (TYPE_VOLET ==1 ) {
        str_nom_typ ="(Bubendorf)";
      }
      if (TYPE_VOLET ==2 ) {
        str_nom_typ ="(Somfy + Stop)";
      }
      Str_Rolling = String(ROLLING_CODE); 
      Str_add_volet = String(ADRESSE_TELECOMMANDE); 
      Str_sens_commande = String(SENS_TELECOMMANDE); 
      Str_automatisation = String(VALIDATION_PROG_AUTO); 
      Str_offset_leve = String(OFFSET_MN_SUNSHINE); 
      Str_offset_descente = String(OFFSET_MN_SUNSET); 
      Str_type_volet = String(TYPE_VOLET);

      PAGE_ACCEUIL =  String("<style>.styled {border: 0;line-height: 2.5;padding:0 20px;font-size:2rem;text-align:center;color: #fff;text-shadow:1px 1px 1px #000;border-radius:10px;background-color:rgba(220, 0, 0, 1);background-image:linear-gradient(to top left,rgba(0, 0, 0, .2),rgba(0, 0, 0, .2) 30%,rgba(0, 0, 0, 0));box-shadow:inset 2px 2px 3px rgba(255, 255, 255, .6),inset -2px -2px 3px rgba(0, 0, 0, .6);}.styled:hover{background-color:rgba(255, 0, 0, 1);}.styled:active{box-shadow:inset -2px -2px 3px rgba(255, 255, 255, .6),inset 2px 2px 3px rgba(0, 0, 0, .6);} input, textarea {  font: 1em sans-serif;width: 100px; box-sizing: border-box;border: 1px solid #999;}</style>");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("<html lang=\"en\"><head><title>Configuration des volets</title> </head><html><head><meta name = \"viewport\" content = \"width = device-width, initial-scale = 1.0, maximum-scale = 1.0, user-scalable=0\"><title>Configuration volets</title></head><h1>Configuration volets</h1>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("<body><a  class=\"favorite styled\" href=\"increment_volet\"> <button> + </button> </a></body><br><form  name=\"formulaire_cfg_vol\" action=\"formulaire_cfg_vol\" method=\"get\"> ");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String(" <input  type=\"text\" name=\"Numero_du_volet\" value=" + Str_numero_volet + ">Numero volet");

      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"Type_du_volet\" value=" + Str_type_volet + ">Type 0=Somfy, 1=Bubendorf, 2=Somfy + stop  : " + str_nom_typ );
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"Nom_du_volet\" value=" + NOM_DU_VOLET + ">Nom du volet"); 

      if (TYPE_VOLET == 0 || TYPE_VOLET == 2) {
          PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"Rolling_code\" value=" + Str_Rolling + ">Rolling code");
          PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"adresse_volet\" value=" + Str_add_volet + ">Adresse volet");
      }
      else
      {
          PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"Rolling_code\" value=" + Str_Rolling + ">Pin volet up");
          PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"adresse_volet\" value=" + Str_add_volet + ">Pin volet down");
      }
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"sens_volet\" value=" + Str_sens_commande + ">Sens 0=normal 1=inverse");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"automatisation_volet\" value=" + Str_automatisation + ">Automatisation 0=non, 1=complete, 2 descente , 3 = monte");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"nb_min_avant\" value=" + Str_offset_leve + ">Lever # minutes avant/apres soleil");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><input  type=\"text\" name=\"nb_min_apres\" value=" + Str_offset_descente + ">Descendre # minutes avant/apres soleil");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  <br><br><br><input  class=\"favorite styled\"  type=\"submit\" value=\"Ok\">");
      PAGE_ACCEUIL =  PAGE_ACCEUIL + String("  </form> </html>");
   }   

}
//-------------------------------------------------------   traite_tous_les_volets ------------------------------------------------------------------------------------------------------------------------------

void    traite_tous_les_volets(int sens_volet){

   // on traite tous les volets, si sens_volet = 0 on descend que ceux qui acceptent la descente automatique  , si sens_volet = 1, on ne monte que ceux qui acceptent la monte automatique
                
        for (int compteur = 0; compteur < NB_VOLETS_DECLARES ; compteur ++ ) {
              VOLET_NUMBER = compteur;
              lire_volet(compteur,true);
              if (  sens_volet == 0 ){  // descente
                    // on verifie qu on accepte la descente 
                    if (VALIDATION_PROG_AUTO == 1 || VALIDATION_PROG_AUTO == 2 ) {
                        descendre_volet(VOLET_NUMBER);
                    }
              }
              else
              {
                    // on verifie qu on accepte la monte
                    if (VALIDATION_PROG_AUTO == 1 || VALIDATION_PROG_AUTO == 3 ) {
                        monter_volet(VOLET_NUMBER);
                    }
              }
              delay(500);
        }
}


// -----------------------------------
//   automatisation
// -----------------------------------


//-------------------------------------------------------   traite_automatisation ------------------------------------------------------------------------------------------------------------------------------


void traite_automatisation(){

unsigned long minute_since_00;
int lever_volet_prog ;
int descente_volet_prog;
           
  minute_since_00 = hour()*60 + minute();

  if ( DATE_TRAITE == day()){  // si on ne change pas de jour, ca veut dire qu'on peut essayer de traiter
    for(int compteur=0;compteur<NB_VOLETS_DECLARES;compteur++){   // on verifie tous les volets declares
      //
      VOLET_NUMBER = compteur;
      // matin
      if (VOLET_TRAITE_MATIN[VOLET_NUMBER] == false ) {  // si le volet est deja traite pour le matin pas la peine de regarder

        lire_volet( VOLET_NUMBER, true );
        lever_volet_prog = get_Sunrise() + OFFSET_MN_SUNSHINE;
       
        if ( lever_volet_prog <= HEURE_MINI_MONTE_VOLETS ) {  // on verifie que l'heure programéee est bien dans la plage mini maxi de monte des volets.
          lever_volet_prog = HEURE_MINI_MONTE_VOLETS+1;
        }
        if ( lever_volet_prog >= HEURE_MAXI_MONTE_VOLETS ) {  // on verifie que l'heure programéee est bien dans la plage mini maxi de monte des volets.
          lever_volet_prog = HEURE_MAXI_MONTE_VOLETS-1;
        }          
        if (minute_since_00 >= HEURE_MINI_MONTE_VOLETS  && minute_since_00 <= HEURE_MAXI_MONTE_VOLETS) {  // on est bien dans la plage d'intervention du matin

          if ( minute_since_00 > lever_volet_prog ){  // on a depasse l'heure de monte du volet par rapport au soleil
            if ( VALIDATION_PROG_AUTO == 1 || VALIDATION_PROG_AUTO == 3) {  // ce volet accepte la monte automatique
               Serial.print("-----------> Automatique Monter volet : "); Serial.print(VOLET_NUMBER); Serial.print("    heure : "); Serial.print(hour()); Serial.print(":");Serial.println(minute());
               delay(500);
               monter_volet (VOLET_NUMBER);
               VOLET_TRAITE_MATIN[VOLET_NUMBER]=true;
            }
          }
        }
      } // fin du matin
      
       // soir
      if (VOLET_TRAITE_SOIR[VOLET_NUMBER] == false ) {  // si le volet est deja traite pour le matin pas la peine de regarder
        lire_volet( VOLET_NUMBER, true );
        descente_volet_prog = get_Sunset()+ OFFSET_MN_SUNSET;

        if ( descente_volet_prog <= HEURE_MINI_DESCENTE_VOLETS ) {  // on verifie que l'heure programéee est bien dans la plage mini maxi de monte des volets.
          descente_volet_prog = HEURE_MINI_DESCENTE_VOLETS+1;
        }
        if ( descente_volet_prog >= HEURE_MAXI_DESCENTE_VOLETS ) {  // on verifie que l'heure programéee est bien dans la plage mini maxi de monte des volets.
          descente_volet_prog = HEURE_MAXI_DESCENTE_VOLETS-1;
        }  
      
        if (minute_since_00 >= HEURE_MINI_DESCENTE_VOLETS  && minute_since_00 <= HEURE_MAXI_DESCENTE_VOLETS) {  // on est bien dans la plage d'intervention du matin

          if ( minute_since_00 > descente_volet_prog ){  // on a depasse l'heure de descentee du volet par rapport au soleil
            if ( VALIDATION_PROG_AUTO == 1 || VALIDATION_PROG_AUTO == 2) {  // ce volet accepte la descente automatique
               Serial.print("-----------> Automatique Descendre volet : "); Serial.print(VOLET_NUMBER); Serial.print("    heure : "); Serial.print(hour()); Serial.print(":");Serial.println(minute());
               delay(500);
               descendre_volet (VOLET_NUMBER);
               VOLET_TRAITE_SOIR[VOLET_NUMBER]=true;
            }
          }
        }
      } // fin du soir
        
    } // fin de boucle sur tous les volets         
  }
  else  // si on change de jour, ca veut dire qu'on est le matin à 00:00 et un chouiaa , on initialise toutes les variables d'automatisme
  {
    // initialisation des variables pour la journée 
    for(int compteur=0;compteur<NB_VOLETS_DECLARES;compteur++){
        VOLET_TRAITE_MATIN[compteur]=0;
        VOLET_TRAITE_SOIR[compteur]=0;
    }
       DATE_TRAITE = day();
  }
}


//-------------------------------------------------------   affiche_menu ------------------------------------------------------------------------------------------------------------------------------

void   affiche_menu(){
    
  Serial.print("Volet courant : ");Serial.println(VOLET_NUMBER);
  Serial.println(" ");
  Serial.println(" ");
  Serial.println("vxxx pour selectionner le volet avec xxx = numero du volet et xxx <= 15");
  Serial.println("p pour appairer le volet courant");
  Serial.println("m pour monter le volet courant");
  Serial.println("d pour descendre le volet courant");
  Serial.println("s pour stopper le volet courant");
  Serial.println("i pour initialiser les parametres du volet courant");
  Serial.println("l pour lire les parametres du volet courant");
  Serial.println("gl pour lire les paramètres généraux");
  Serial.println("ge pour ecrire les paramètres généraux");
  Serial.println("gx pour initialiser l'EEPROM");
  Serial.println("h pour verifier l'heure de l4ESP8266 ainsi que les heures de lever et coucher du soleil");
  Serial.println("? afficher ce menu");
}
